package com.putnam.jboss.issues;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Hashtable;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

/**
 * Client to demonstrate the issue that when sending a 1M message, the publisher fails if the min-large-message-size
 * configuration is set to <tt>10485760</tt>.
 * The publisher fails with an exception:
 * <p>
 * {@code
 * javax.jms.JMSException
    at org.hornetq.core.protocol.core.impl.ChannelImpl.sendBlocking(ChannelImpl.java:287)
    at org.hornetq.core.client.impl.ClientProducerImpl.doSend(ClientProducerImpl.java:285)
    at org.hornetq.core.client.impl.ClientProducerImpl.send(ClientProducerImpl.java:139)
    at org.hornetq.jms.client.HornetQMessageProducer.doSend(HornetQMessageProducer.java:451)
    at org.hornetq.jms.client.HornetQMessageProducer.send(HornetQMessageProducer.java:199)
    at com.putnam.jboss.issues.Send1MMsgFails.run(Send1MMsgFails.java:132)Closed producer...
 * }
 * <p>
 * @author aengineer
 *
 */
public class Send1MMsgFails {
    private static final String logPrefix = "QueueSend::";

    /*
     * HornetQ
     */
    private final static String JNDI_FACTORY ="org.jnp.interfaces.NamingContextFactory";
    private final static String PROVIDER_URL = "jnp://richmond2.putnaminv.com:6142";
    private final static String JNDI_PRINCIPAL = "";
    private final static String JNDI_PASSWORD = "";
    private final static String JNDI_URL_PKGS = "org.jboss.naming:org.jnp.interfaces";
    private final static String DEST_LOOKUP_NAME = "/queue/ExampleQueue";
    private final static String CF_LOOKUP_NAME = "/ConnectionFactory";
    private final static String DEST_NAME = "ExampleQueue";
    private final static String BROKER_USER = "testuser";
    private final static String BROKER_PASSWORD = "testuser";
    
    
    public static Hashtable<String, String> JNDIEnv = new Hashtable<String, String>();
    static {
        JNDIEnv.put(Context.INITIAL_CONTEXT_FACTORY, JNDI_FACTORY);
        JNDIEnv.put(Context.PROVIDER_URL, PROVIDER_URL);
        JNDIEnv.put(Context.SECURITY_PRINCIPAL, JNDI_PRINCIPAL);
        JNDIEnv.put(Context.SECURITY_CREDENTIALS, JNDI_PASSWORD); 
        JNDIEnv.put(Context.URL_PKG_PREFIXES, JNDI_URL_PKGS);
    }
    
    /**
     * Load all the contents of a file as a STring
     * @param inFile
     * @return
     * @throws Exception
     */
    private String loadFile(File inFile) throws Exception {
        BufferedReader br = new BufferedReader(new FileReader(inFile));
        String aLine = null;
        StringBuffer bodyBuffer = new StringBuffer();
        while ((aLine = br.readLine()) != null) {
            bodyBuffer.append(aLine);
        }
        br.close();
        
        return (bodyBuffer.toString());         
    }
    
    /**
     * 
     */
    private static void usage() {
        System.out.println("Usage: com.putnam.jboss.issues.Send1MMsgFails <data-file-path>");
    }
    
    /**
     * @param args
     */
    public void run(String[] args) {
        String logMsgPrefix = logPrefix + "run::";
        String message = null;
        String filePath = null;
        ConnectionFactory connFactory = null;
        Connection conn = null;
        Session session = null;
        MessageProducer msgProducer = null;
        MessageConsumer msgConsumer = null;
        Destination aDestination = null;
        TextMessage tm = null;
        
        /*
         * Validate that the required arguments were passed in.
         */
        if (args.length < 1) {
            System.err.println(logMsgPrefix + "Invalid number of arguments");
            Send1MMsgFails.usage();
            return;
        } else {
            filePath = args[0];
        }

        File inFile = new File(filePath);
        if (!inFile.exists()) {
            message = "File [" + filePath + "] does not exist.";
            System.err.println(message);
            return;
        } else {
            System.out.println("Will use data from file [" + filePath + "]");
        }
        
            
        try {
            InitialContext ctx = new InitialContext(JNDIEnv);            
            connFactory = (ConnectionFactory) ctx.lookup(CF_LOOKUP_NAME);
            conn = connFactory.createConnection(BROKER_USER, BROKER_PASSWORD);
            conn.setClientID("JBossPOC.SelectorTest");
            session = conn.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            aDestination = (Destination) ctx.lookup(DEST_LOOKUP_NAME);
            msgProducer = session.createProducer(aDestination);
            System.out.println("Using consumer destination: [" + DEST_LOOKUP_NAME + "]");
            conn.start();
            
            String fileContents = loadFile(inFile);
            /* Send 1 message */
            session = conn.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            msgProducer = session.createProducer(aDestination);
            tm = session.createTextMessage();
            tm.setText(fileContents);
            msgProducer.send(tm);
            System.out.println ("Sent message with id [" + tm.getJMSMessageID() + "]");
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("An exception occurred");
            return;
        } finally {
            try {
                if (msgProducer != null) {
                    msgProducer.close();
                    msgProducer = null;
                    System.out.println("Closed producer...");
                }                
                if (msgConsumer != null) {
                    msgConsumer.close();
                    msgConsumer = null;
                    System.out.println("Closed consumer...");
                }
                if (session != null) {
                    session.close();
                    session = null;
                    System.out.println("Closed session...");
                }
                if (conn != null) {
                    conn.close();
                    conn = null;
                    System.out.println("Closed connection...");
                }
            } catch (JMSException jmse2) {
                jmse2.printStackTrace();
                System.err.println("An exception occurred when closing objects: " + jmse2.getMessage());
            }
        }
    }
    

    
    /**
     * main() method.
     * 
     */
    public static void main(String[] args) throws Exception {
        Send1MMsgFails client = new Send1MMsgFails();
        client.run(args);
    }    
}
